@@ -5,7 +5,7 @@ from contextlib import suppress
 
 from PyQt5.QtCore import Qt
 from PyQt5.QtGui import QImage, QPixmap
-from PyQt5.QtWidgets import QFrame, QVBoxLayout, QLabel
+from PyQt5.QtWidgets import QFrame, QVBoxLayout, QLabel, QApplication
 from requests.exceptions import RequestException
 
 from feeluown.utils import aio
@@ -17,6 +17,7 @@ from feeluown.models import reverse, ModelType
 
 from feeluown.gui.helpers import async_run, BgTransparentMixin, \
     disconnect_slots_if_has, fetch_cover_wrapper
+from feeluown.gui.widgets.menu import SongMenuInitializer
 from feeluown.gui.widgets.imglist import ImgListView
 from feeluown.gui.widgets.album import AlbumListModel, AlbumListView, \
     AlbumFilterProxyModel
@@ -443,15 +444,13 @@ class TableContainer(QFrame, BgTransparentMixin):
 
         model = self.songs_table.model()
         # FIXME: think about a more elegant way
-        reader = model.sourceModel()._reader
-        if reader is not None:
-            if reader.count is not None:
-                task = task_spec.bind_blocking_io(reader.readall)
-                self.toolbar.enter_state_playall_start()
-                task.add_done_callback(reader_readall_cb)
-                return
-        songs = model.sourceModel().songs
-        self._app.player.play_songs(songs=songs)
+        reader = model.sourceModel().reader
+        if reader is not None and reader.count is not None:
+            task = task_spec.bind_blocking_io(reader.readall)
+            self.toolbar.enter_state_playall_start()
+            task.add_done_callback(reader_readall_cb)
+            return
+        assert False, 'The play_all_btn should be hide in this page'
 
     async def show_model(self, model):
         model = self._app.library.cast_model_to_v1(model)
@@ -495,15 +494,14 @@ class TableContainer(QFrame, BgTransparentMixin):
             self._app.playlist.add(song)
 
     def _songs_table_about_to_show_menu(self, ctx):
-        add_action = ctx['add_action']
         models = ctx['models']
         if not models or models[0].meta.model_type != ModelType.song:
             return
 
+        menu = ctx['menu']
         song = models[0]
-        goto = self._app.browser.goto
-
-        add_action('歌曲详情', lambda *args: goto(model=song, path='/explore'))
+        menu.addSeparator()
+        SongMenuInitializer(self._app, song).apply(menu)
 
     async def _on_songs_table_activated(self, index):
         """
@@ -513,7 +511,18 @@ class TableContainer(QFrame, BgTransparentMixin):
 
         song = index.data(Qt.UserRole)
         if index.column() == Column.song:
-            self.songs_table.play_song_needed.emit(song)
+            # .. versionadded:: 3.7.11
+            #    Reset playlist with songs in table if Alt key is pressed.
+            if QApplication.keyboardModifiers() & Qt.AltModifier:
+                model = self.songs_table.model()
+                if isinstance(model, SongFilterProxyModel):
+                    model = model.sourceModel()
+                    if isinstance(model, SongsTableModel):
+                        reader = model.reader
+                        songs = await aio.run_fn(reader.readall)
+                        self._app.playlist.clear()
+                        self._app.playlist.init_from(songs)
+            self._app.player.play_song(song)
         else:
             try:
                 song = await aio.run_in_executor(
