@@ -154,19 +154,11 @@ class SongListView(ItemViewNoScrollMixin, QListView):
         self.play_song_needed.emit(index.data(Qt.UserRole))
 
 
-class SongsTableModel(QAbstractTableModel, ReaderFetchMoreMixin):
-    def __init__(self, source_name_map=None, reader=None, parent=None):
-        """
-
-        :param songs: 歌曲列表
-        :param songs_g: 歌曲列表生成器（当歌曲列表生成器不为 None 时，忽略 songs 参数）
-        """
+class BaseSongsTableModel(QAbstractTableModel):
+    def __init__(self, source_name_map=None, parent=None):
         super().__init__(parent)
-        self._reader = reader
-        self._fetch_more_step = 30
-        self._items = []
-        self._is_fetching = False
 
+        self._items = []
         self._source_name_map = source_name_map or {}
 
     def removeRows(self, row, count, parent=QModelIndex()):
@@ -286,6 +278,23 @@ class SongsTableModel(QAbstractTableModel, ReaderFetchMoreMixin):
             return ModelMimeData(model)
 
 
+class SongsTableModel(BaseSongsTableModel, ReaderFetchMoreMixin):
+    def __init__(self, source_name_map=None, reader=None, parent=None):
+        """
+
+        :param songs: 歌曲列表
+        :param songs_g: 歌曲列表生成器（当歌曲列表生成器不为 None 时，忽略 songs 参数）
+        """
+        super().__init__(source_name_map, parent)
+        self._reader = reader
+        self._fetch_more_step = 30
+        self._is_fetching = False
+
+    @property
+    def reader(self):
+        return self._reader
+
+
 class SongFilterProxyModel(QSortFilterProxyModel):
     def __init__(self, parent=None, text=''):
         super().__init__(parent)
@@ -511,9 +520,13 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
             menu.addAction(action)
             action.triggered.connect(lambda: callback(models))
 
-        # .. versionadded: v3.7
+        # .. versionadded: 3.7
         #   The context key *models*
-        self.about_to_show_menu.emit({'add_action': add_action, 'models': models})
+        # .. versionadded: 3.7.11
+        #   The context key *menu*
+        self.about_to_show_menu.emit({'add_action': add_action,
+                                      'menu': menu,
+                                      'models': models})
         menu.exec(event.globalPos())
 
     def _add_to_playlist(self, indexes):
