@@ -3,6 +3,7 @@ from .base_player import State
 from .mpvplayer import MpvPlayer as Player
 from .playlist import PlaylistMode, Playlist
 from .fm import FM
+from .radio import SongRadio
 
 
 __all__ = (
@@ -11,6 +12,7 @@ __all__ = (
 
     'FM',
     'PlaylistMode',
+    'SongRadio',
 
     'Player',
     'Playlist',
