@@ -31,19 +31,9 @@ class PlaylistMode(IntEnum):
 
     In FM mode, playlist's playback_mode is unchangeable, it will
     always be sequential. When playlist has no more song,
-    the playlist hopes someone(we call it ``FMPlaylist`` here) will:
+    the playlist hopes someone(we call it ``FM`` here) will:
     1. catch the ``eof_reached`` signal
     2. add news songs to playlist by using ``fm_add`` method
-    3. call ``next`` method to resume the player
-
-    **How to enter FM mode?**
-
-    Only FMPlaylist can(should) make playlist enter FM mode, it should
-    do following things:
-    1. clear the playlist
-    2. change playlist mode to FM
-    3. add several songs to playlist
-    4. resume the player with the first song
 
     **When will playlist exit FM mode?**
 
@@ -112,6 +102,11 @@ class Playlist:
         #: When watch mode is on, playlist try to play the mv/video of the song
         self.watch_mode = False
 
+        # .. versionadded:: 3.7.11
+        #    The *songs_removed* and *songs_added* signal.
+        self.songs_removed = Signal()  # (index, count)
+        self.songs_added = Signal()  # (index, count)
+
     @property
     def mode(self):
         return self._mode
@@ -122,7 +117,6 @@ class Playlist:
         if self._mode is not mode:
             if mode is PlaylistMode.fm:
                 self.playback_mode = PlaybackMode.sequential
-            self.clear()
             # we should change _mode at the very end
             self._mode = mode
             self.mode_changed.emit(mode)
@@ -146,6 +140,8 @@ class Playlist:
         if song in self._songs:
             return
         self._songs.append(song)
+        length = len(self._songs)
+        self.songs_added.emit(length-1, 1)
 
     def add(self, song):
         """add song to playlist
@@ -171,10 +167,11 @@ class Playlist:
         if song in self._songs:
             return
         if self._current_song is None:
-            self._songs.append(song)
+            self._add(song)
         else:
             index = self._songs.index(self._current_song)
             self._songs.insert(index + 1, song)
+            self.songs_added.emit(index + 1, 1)
 
     def remove(self, song):
         """Remove song from playlist. O(n)
@@ -182,7 +179,11 @@ class Playlist:
         If song is current song, remove the song and play next. Otherwise,
         just remove it.
         """
-        if song in self._songs:
+        try:
+            index = self._songs.index(song)
+        except ValueError:
+            logger.debug('Remove failed: {} not in playlist'.format(song))
+        else:
             if self._current_song is None:
                 self._songs.remove(song)
             elif song == self._current_song:
@@ -198,10 +199,8 @@ class Playlist:
                     self.current_song = next_song
             else:
                 self._songs.remove(song)
+            self.songs_removed.emit(index, 1)
             logger.debug('Remove {} from player playlist'.format(song))
-        else:
-            logger.debug('Remove failed: {} not in playlist'.format(song))
-
         if song in self._bad_songs:
             self._bad_songs.remove(song)
 
@@ -225,12 +224,17 @@ class Playlist:
         # since we will call songs.clear method during playlist clearing,
         # we need to deepcopy songs object here.
         self._songs = DedupList(copy.deepcopy(songs))
+        if self._songs:
+            self.songs_added.emit(0, len(self._songs))
 
     def clear(self):
         """remove all songs from playlists"""
         if self.current_song is not None:
             self.current_song = None
+        length = len(self._songs)
         self._songs.clear()
+        if length > 0:
+            self.songs_removed.emit(0, length)
         self._bad_songs.clear()
 
     def list(self):
@@ -332,15 +336,16 @@ class Playlist:
             previous_song = self._get_good_song(base=current_index - 1, direction=-1)
         return previous_song
 
-    def next(self):
+    def next(self) -> Optional[asyncio.Task]:
         if self.next_song is None:
             self.eof_reached.emit()
+            return None
         else:
-            self.current_song = self.next_song
+            return self.set_current_song(self.next_song)
 
-    def previous(self):
+    def previous(self) -> Optional[asyncio.Task]:
         """return to the previous song in playlist"""
-        self.current_song = self.previous_song
+        return self.set_current_song(self.previous_song)
 
     @property
     def current_song(self) -> Optional[SongProtocol]:
@@ -352,6 +357,9 @@ class Playlist:
 
     @current_song.setter
     def current_song(self, song: Optional[SongProtocol]):
+        self.set_current_song(song)
+
+    def set_current_song(self, song) -> Optional[asyncio.Task]:
         """设置当前歌曲，将歌曲加入到播放列表，并发出 song_changed 信号
 
         .. note::
@@ -359,10 +367,13 @@ class Playlist:
             该方法理论上只应该被 Player 对象调用。
 
         if song has not valid media, we find a replacement in other providers
+
+        .. versionadded:: 3.7.11
+           The method is added to replace current_song.setter.
         """
         if song is None:
             self.pure_set_current_song(None, None)
-            return
+            return None
 
         if self.mode is PlaylistMode.fm and song not in self._songs:
             self.mode = PlaylistMode.normal
@@ -370,7 +381,7 @@ class Playlist:
         # FIXME(cosven): `current_song.setter` depends on app.task_mgr and app.library,
         # which make it hard to test.
         task_spec = self._app.task_mgr.get_or_create('set-current-song')
-        task_spec.bind_coro(self.a_set_current_song(song))
+        return task_spec.bind_coro(self.a_set_current_song(song))
 
     async def a_set_current_song(self, song):
         song_str = f'{song.source}:{song.title_display} - {song.artists_name_display}'
@@ -397,6 +408,7 @@ class Playlist:
                     if song in self._songs and standby not in self._songs:
                         index = self._songs.index(song)
                         self._songs.insert(index + 1, standby)
+                        self.songs_added.emit(index + 1, 1)
                     # NOTE: a_list_song_standby ensure that the song.url is not empty
                     # FIXME: maybe a_list_song_standby should return media directly
                     self.pure_set_current_song(standby, media)
